@@ -1,7 +1,7 @@
 /* -*- C++ -*- */
 /*
    Copyright (c) 2002, 2011, Oracle and/or its affiliates.
-   Copyright (c) 2020, MariaDB
+   Copyright (c) 2020, 2022, MariaDB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -47,6 +47,14 @@ class sp_instr;
 class sp_instr_opt_meta;
 class sp_instr_jump_if_not;
 
+/**
+  Number of PSI_statement_info instruments
+  for internal stored programs statements.
+*/
+#ifdef HAVE_PSI_INTERFACE
+void init_sp_psi_keys(void);
+#endif
+
 /*************************************************************************/
 
 /**
@@ -109,15 +117,14 @@ public:
           bool use_explicit_name)
     : Database_qualified_name(db, name), m_explicit_name(use_explicit_name)
   {
-    if (lower_case_table_names && m_db.str)
+    if (lower_case_table_names && m_db.length)
       m_db.length= my_casedn_str(files_charset_info, (char*) m_db.str);
   }
 
   /** Create temporary sp_name object from MDL key. Store in qname_buff */
   sp_name(const MDL_key *key, char *qname_buff);
 
-  ~sp_name()
-  {}
+  ~sp_name() = default;
 };
 
 
@@ -174,6 +181,11 @@ public:
   const Sp_handler *m_handler;
   uint m_flags;                 // Boolean attributes of a stored routine
 
+  /**
+    Instrumentation interface for SP.
+  */
+  PSI_sp_share *m_sp_share;
+
   Column_definition m_return_field_def; /**< This is used for FUNCTIONs only. */
 
   const char *m_tmp_query;	///< Temporary pointer to sub query string
@@ -492,6 +504,18 @@ private:
                                                sp_assignment_lex *param_lex,
                                                Item_args *parameters);
 
+  bool bind_input_param(THD *thd,
+                        Item *arg_item,
+                        uint arg_no,
+                        sp_rcontext *nctx,
+                        bool is_function);
+
+  bool bind_output_param(THD *thd,
+                         Item *arg_item,
+                         uint arg_no,
+                         sp_rcontext *octx,
+                         sp_rcontext *nctx);
+
 public:
   /**
     Generate a code for an "OPEN cursor" statement.
@@ -548,7 +572,7 @@ public:
   { return m_flags & MODIFIES_DATA; }
 
   inline uint instructions()
-  { return m_instr.elements; }
+  { return (uint)m_instr.elements; }
 
   inline sp_instr *
   last_instruction()
@@ -612,6 +636,23 @@ public:
     DBUG_RETURN(false);
   }
 
+  /**
+    Iterate through the LEX stack from the top (the newest) to the bottom
+    (the oldest) and find the one that contains a non-zero spname.
+    @returns - the address of spname, or NULL of no spname found.
+  */
+  const sp_name *find_spname_recursive()
+  {
+    uint count= m_lex.elements;
+    for (uint i= 0; i < count; i++)
+    {
+      const LEX *tmp= m_lex.elem(count - i - 1);
+      if (tmp->spname)
+        return tmp->spname;
+    }
+    return NULL;
+  }
+
   /// Put the instruction on the backpatch list, associated with the label.
   int
   push_backpatch(THD *thd, sp_instr *, sp_label *);
@@ -856,6 +897,8 @@ public:
     return NULL;
   }
 
+  virtual void init_psi_share();
+
 protected:
 
   MEM_ROOT *m_thd_root;		///< Temp. store for thd's mem_root
@@ -931,9 +974,9 @@ public:
   public:
     LexList() { elements= 0; }
     // Find a package routine by a non qualified name
-    LEX *find(const LEX_CSTRING &name, stored_procedure_type type);
+    LEX *find(const LEX_CSTRING &name, enum_sp_type type);
     // Find a package routine by a package-qualified name, e.g. 'pkg.proc'
-    LEX *find_qualified(const LEX_CSTRING &name, stored_procedure_type type);
+    LEX *find_qualified(const LEX_CSTRING &name, enum_sp_type type);
     // Check if a routine with the given qualified name already exists
     bool check_dup_qualified(const LEX_CSTRING &name, const Sp_handler *sph)
     {
@@ -990,6 +1033,7 @@ public:
            m_routine_implementations.push_back(lex, &main_mem_root);
   }
   sp_package *get_package() { return this; }
+  void init_psi_share();
   bool is_invoked() const
   {
     /*
@@ -1025,8 +1069,9 @@ public:
     Query_arena(thd->lex->sphead->get_main_mem_root(), STMT_INITIALIZED_FOR_SP)
   { }
   ~sp_lex_cursor() { free_items(); }
-  void cleanup_stmt() { }
-  Query_arena *query_arena() { return this; }
+  virtual bool cleanup_stmt(bool /*restore_set_statement_vars*/) override
+  { return false; }
+  Query_arena *query_arena() override { return this; }
   bool validate()
   {
     DBUG_ASSERT(sql_command == SQLCOM_SELECT);
@@ -1156,6 +1201,7 @@ public:
   {
     m_ip= dst;
   }
+  virtual PSI_statement_info* get_psi_info() = 0;
 
 }; // class sp_instr : public Sql_alloc
 
@@ -1269,8 +1315,7 @@ public:
     m_query.length= 0;
   }
 
-  virtual ~sp_instr_stmt()
-  {};
+  virtual ~sp_instr_stmt() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1282,6 +1327,10 @@ private:
 
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
+
 }; // class sp_instr_stmt : public sp_instr
 
 
@@ -1301,8 +1350,7 @@ public:
       m_lex_keeper(lex, lex_resp)
   {}
 
-  virtual ~sp_instr_set()
-  {}
+  virtual ~sp_instr_set() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1316,6 +1364,10 @@ protected:
   uint m_offset;		///< Frame offset
   Item *m_value;
   sp_lex_keeper m_lex_keeper;
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_set : public sp_instr
 
 
@@ -1341,8 +1393,7 @@ public:
       m_field_offset(field_offset)
   {}
 
-  virtual ~sp_instr_set_row_field()
-  {}
+  virtual ~sp_instr_set_row_field() = default;
 
   virtual int exec_core(THD *thd, uint *nextp);
 
@@ -1384,8 +1435,7 @@ public:
       m_field_name(field_name)
   {}
 
-  virtual ~sp_instr_set_row_field_by_name()
-  {}
+  virtual ~sp_instr_set_row_field_by_name() = default;
 
   virtual int exec_core(THD *thd, uint *nextp);
 
@@ -1411,8 +1461,7 @@ public:
       value(val), m_lex_keeper(lex, TRUE)
   {}
 
-  virtual ~sp_instr_set_trigger_field()
-  {}
+  virtual ~sp_instr_set_trigger_field() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1424,6 +1473,10 @@ private:
   Item_trigger_field *trigger_field;
   Item *value;
   sp_lex_keeper m_lex_keeper;
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_trigger_field : public sp_instr
 
 
@@ -1451,8 +1504,7 @@ public:
       m_dest(dest), m_cont_dest(0), m_optdest(0), m_cont_optdest(0)
   {}
 
-  virtual ~sp_instr_opt_meta()
-  {}
+  virtual ~sp_instr_opt_meta() = default;
 
   virtual void set_destination(uint old_dest, uint new_dest)
     = 0;
@@ -1481,8 +1533,7 @@ public:
     : sp_instr_opt_meta(ip, ctx, dest)
   {}
 
-  virtual ~sp_instr_jump()
-  {}
+  virtual ~sp_instr_jump() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1510,6 +1561,9 @@ public:
       m_dest= new_dest;
   }
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_jump : public sp_instr_opt_meta
 
 
@@ -1530,8 +1584,7 @@ public:
       m_lex_keeper(lex, TRUE)
   {}
 
-  virtual ~sp_instr_jump_if_not()
-  {}
+  virtual ~sp_instr_jump_if_not() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1561,6 +1614,9 @@ private:
   Item *m_expr;			///< The condition
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_jump_if_not : public sp_instr_jump
 
 
@@ -1575,20 +1631,11 @@ public:
     : sp_instr(ip, ctx)
   {}
 
-  virtual ~sp_instr_preturn()
-  {}
+  virtual ~sp_instr_preturn() = default;
 
-  virtual int execute(THD *thd, uint *nextp)
-  {
-    DBUG_ENTER("sp_instr_preturn::execute");
-    *nextp= UINT_MAX;
-    DBUG_RETURN(0);
-  }
+  virtual int execute(THD *thd, uint *nextp);
 
-  virtual void print(String *str)
-  {
-    str->append(STRING_WITH_LEN("preturn"));
-  }
+  virtual void print(String *str);
 
   virtual uint opt_mark(sp_head *sp, List<sp_instr> *leads)
   {
@@ -1596,6 +1643,9 @@ public:
     return UINT_MAX;
   }
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_preturn : public sp_instr
 
 
@@ -1612,8 +1662,7 @@ public:
       m_lex_keeper(lex, TRUE)
   {}
 
-  virtual ~sp_instr_freturn()
-  {}
+  virtual ~sp_instr_freturn() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1633,6 +1682,9 @@ protected:
   const Type_handler *m_type_handler;
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_freturn : public sp_instr
 
 
@@ -1698,6 +1750,9 @@ private:
   // debug version only). It's used in print().
   uint m_frame;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_hpush_jump : public sp_instr_jump
 
 
@@ -1712,8 +1767,7 @@ public:
     : sp_instr(ip, ctx), m_count(count)
   {}
 
-  virtual ~sp_instr_hpop()
-  {}
+  virtual ~sp_instr_hpop() = default;
 
   void update_count(uint count)
   {
@@ -1728,6 +1782,9 @@ private:
 
   uint m_count;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_hpop : public sp_instr
 
 
@@ -1743,8 +1800,7 @@ public:
     m_frame(ctx->current_var_count())
   {}
 
-  virtual ~sp_instr_hreturn()
-  {}
+  virtual ~sp_instr_hreturn() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1762,12 +1818,14 @@ private:
 
   uint m_frame;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_hreturn : public sp_instr_jump
 
 
 /** This is DECLARE CURSOR */
-class sp_instr_cpush : public sp_instr,
-                       public sp_cursor
+class sp_instr_cpush : public sp_instr, public sp_cursor
 {
   sp_instr_cpush(const sp_instr_cpush &); /**< Prevent use of these */
   void operator=(sp_instr_cpush &);
@@ -1778,24 +1836,27 @@ public:
     : sp_instr(ip, ctx), m_lex_keeper(lex, TRUE), m_cursor(offset)
   {}
 
-  virtual ~sp_instr_cpush()
-  {}
+  virtual ~sp_instr_cpush() = default;
 
-  virtual int execute(THD *thd, uint *nextp);
+  int execute(THD *thd, uint *nextp) override;
 
-  virtual void print(String *str);
+  void print(String *str) override;
 
   /**
     This call is used to cleanup the instruction when a sensitive
     cursor is closed. For now stored procedures always use materialized
     cursors and the call is not used.
   */
-  virtual void cleanup_stmt() { /* no op */ }
+  virtual bool cleanup_stmt(bool /*restore_set_statement_vars*/) override
+  { return false; }
 private:
 
   sp_lex_keeper m_lex_keeper;
   uint m_cursor;                /**< Frame offset (for debugging) */
 
+public:
+  PSI_statement_info* get_psi_info() override { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cpush : public sp_instr
 
 
@@ -1810,8 +1871,7 @@ public:
     : sp_instr(ip, ctx), m_count(count)
   {}
 
-  virtual ~sp_instr_cpop()
-  {}
+  virtual ~sp_instr_cpop() = default;
 
   void update_count(uint count)
   {
@@ -1826,6 +1886,9 @@ private:
 
   uint m_count;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cpop : public sp_instr
 
 
@@ -1840,8 +1903,7 @@ public:
     : sp_instr(ip, ctx), m_cursor(c)
   {}
 
-  virtual ~sp_instr_copen()
-  {}
+  virtual ~sp_instr_copen() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1853,6 +1915,9 @@ private:
 
   uint m_cursor;		///< Stack index
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_copen : public sp_instr_stmt
 
 
@@ -1875,11 +1940,14 @@ public:
       m_cursor(coffs),
       m_var(voffs)
   {}
-  virtual ~sp_instr_cursor_copy_struct()
-  {}
+  virtual ~sp_instr_cursor_copy_struct() = default;
   virtual int execute(THD *thd, uint *nextp);
   virtual int exec_core(THD *thd, uint *nextp);
   virtual void print(String *str);
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 };
 
 
@@ -1894,8 +1962,7 @@ public:
     : sp_instr(ip, ctx), m_cursor(c)
   {}
 
-  virtual ~sp_instr_cclose()
-  {}
+  virtual ~sp_instr_cclose() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1905,6 +1972,9 @@ private:
 
   uint m_cursor;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cclose : public sp_instr
 
 
@@ -1921,8 +1991,7 @@ public:
     m_varlist.empty();
   }
 
-  virtual ~sp_instr_cfetch()
-  {}
+  virtual ~sp_instr_cfetch() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1939,6 +2008,9 @@ private:
   List<sp_variable> m_varlist;
   bool m_error_on_no_data;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_cfetch : public sp_instr
 
 /*
@@ -1957,12 +2029,15 @@ public:
   sp_instr_agg_cfetch(uint ip, sp_pcontext *ctx)
     : sp_instr(ip, ctx){}
 
-  virtual ~sp_instr_agg_cfetch()
-  {}
+  virtual ~sp_instr_agg_cfetch() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
   virtual void print(String *str);
+
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_agg_cfetch : public sp_instr
 
 
@@ -1979,8 +2054,7 @@ public:
     : sp_instr(ip, ctx), m_errcode(errcode)
   {}
 
-  virtual ~sp_instr_error()
-  {}
+  virtual ~sp_instr_error() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -1996,6 +2070,9 @@ private:
 
   int m_errcode;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_error : public sp_instr
 
 
@@ -2010,8 +2087,7 @@ public:
       m_lex_keeper(lex, TRUE)
   {}
 
-  virtual ~sp_instr_set_case_expr()
-  {}
+  virtual ~sp_instr_set_case_expr() = default;
 
   virtual int execute(THD *thd, uint *nextp);
 
@@ -2035,6 +2111,9 @@ private:
   Item *m_case_expr;
   sp_lex_keeper m_lex_keeper;
 
+public:
+  virtual PSI_statement_info* get_psi_info() { return & psi_info; }
+  static PSI_statement_info psi_info;
 }; // class sp_instr_set_case_expr : public sp_instr_opt_meta
 
 bool check_show_routine_access(THD *thd, sp_head *sp, bool *full_access);
