@@ -1,5 +1,6 @@
 /* -*- C++ -*- */
 /* Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2009, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +23,6 @@
 #endif
 
 #include "sql_string.h"                         // LEX_STRING
-#include "mysql_com.h"                          // enum_field_types
 #include "field.h"                              // Create_field
 #include "sql_array.h"                          // Dynamic_array
 
@@ -60,7 +60,8 @@ public:
   Spvar_definition field_def;
 
   /// Field-type of the SP-variable.
-  const Type_handler *type_handler() const { return field_def.type_handler(); }
+  const Type_handler *type_handler() const
+    { return field_def.type_handler(); }
 
 public:
   sp_variable(const LEX_CSTRING *name_arg, uint offset_arg)
@@ -261,9 +262,8 @@ public:
   }
   bool eq_name(const LEX_CSTRING *str) const
   {
-    return my_strnncoll(system_charset_info,
-                        (const uchar *) name.str, name.length,
-                        (const uchar *) str->str, str->length) == 0;
+    return system_charset_info->strnncoll(name.str, name.length,
+                                          str->str, str->length) == 0;
   }
 };
 
