@@ -22,10 +22,11 @@
   that is defined in plugin.h
 */
 #define SHOW_always_last SHOW_KEY_CACHE_LONG, \
-            SHOW_LONG_STATUS, SHOW_DOUBLE_STATUS, \
             SHOW_HAVE, SHOW_MY_BOOL, SHOW_HA_ROWS, SHOW_SYS, \
-            SHOW_LONG_NOFLUSH, SHOW_LONGLONG_STATUS, SHOW_UINT32_STATUS, \
-            SHOW_LEX_STRING, SHOW_ATOMIC_COUNTER_UINT32_T
+            SHOW_LONG_NOFLUSH, SHOW_LEX_STRING, SHOW_ATOMIC_COUNTER_UINT32_T, \
+      /* SHOW_*_STATUS must be at the end, SHOW_LONG_STATUS being first */ \
+            SHOW_LONG_STATUS, SHOW_DOUBLE_STATUS, SHOW_LONGLONG_STATUS, \
+            SHOW_UINT32_STATUS
 #include "mariadb.h"
 #undef SHOW_always_last
 
@@ -171,7 +172,7 @@ extern plugin_ref plugin_lock(THD *thd, plugin_ref ptr);
 extern plugin_ref plugin_lock_by_name(THD *thd, const LEX_CSTRING *name,
                                       int type);
 extern void plugin_unlock(THD *thd, plugin_ref plugin);
-extern void plugin_unlock_list(THD *thd, plugin_ref *list, uint count);
+extern void plugin_unlock_list(THD *thd, plugin_ref *list, size_t count);
 extern bool mysql_install_plugin(THD *thd, const LEX_CSTRING *name,
                                  const LEX_CSTRING *dl);
 extern bool mysql_uninstall_plugin(THD *thd, const LEX_CSTRING *name,
@@ -203,4 +204,3 @@ extern void sync_dynamic_session_variables(THD* thd, bool global_lock);
 extern void wsrep_plugins_pre_init();
 extern void wsrep_plugins_post_init();
 #endif /* WITH_WSREP */
-
